@@ -18,6 +18,12 @@ import numpy as np
 
 # Project imports
 
+GROUNDHOG_PLOTTING_CONFIG = {
+    'showLink': True,
+    'plotlyServerURL': "https://github.com/snakesonabrain/groundhog",
+    'linkText': 'Created by groundhog using Plotly!'
+}
+
 PLOTLY_GLOBAL_FONT = dict(family='Century Gothic', size=12, color='#5f5f5f')
 C0 = '#1f77b4'; C1 = '#ff7f0e'; C2 = '#2ca02c'; C3 = '#d62728'; C4 = '#9467bd'; C5 = '#8c564b'; C6 = '#e377c2'; C7 = '#7f7f7f'; C8 = '#bcbd22'; C9 = '#17becf'
 PLOTLY_COLORS = [C0, C1, C2, C3, C4, C5, C6, C7, C8, C9]
@@ -533,5 +539,6 @@ def plot_with_log(x=[[],], z=[[],], names=[[],], showlegends=None,
             _fig['layout']['xaxis%i' % (i + 2)].update(range=xranges[i])
 
     if showfig:
-        iplot(_fig, filename='logplot')
-    return _fig
\ No newline at end of file
+        iplot(_fig, filename='logplot', config=GROUNDHOG_PLOTTING_CONFIG)
+    return _fig
+
